use clap::Parser;
use tokio::{sync::mpsc, task::JoinSet};

mod carla;
mod client;
mod server;

#[derive(Debug, Clone, Parser)]
#[command(version, about, long_about=None)]
struct Args {
    #[arg(long, default_value_t = String::from("127.0.0.1"))]
    bridge_host: String,
    #[arg(long, default_value_t = String::from("127.0.0.1:22752"))]
    carla_host: String,
    #[arg(long)]
    clients: Vec<String>,
    #[arg(long, default_value_t = 10400)]
    carla_recv_port: u16,
    #[arg(long, default_value_t = 10401)]
    carla_send_port: u16,
    #[arg(long, default_value_t = 10402)]
    client_recv_port: u16,
    #[arg(long, default_value_t = 10403)]
    client_send_port: u16,
}

#[tokio::main]
async fn main() {
    let args = Args::parse();
    println!("Starting bridge with args:",);
    dbg!(&args);

    let (from_carla_tx, from_carla_rx) = mpsc::channel(1024);
    let (to_carla_tx, to_carla_rx) = mpsc::channel(1024);

    let mut set = JoinSet::new();
    set.spawn(carla::setup_carla(args.clone(), from_carla_tx));
    set.spawn(client::setup_clients(args.clone(), from_carla_rx));
    set.spawn(server::setup_server(args.clone(), to_carla_tx));
    set.spawn(carla::setup_forwarder(args, to_carla_rx));

    while let Some(res) = set.join_next().await {
        res.unwrap().unwrap();
    }
}
